@@ -25,13 +25,11 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	clientgotesting "k8s.io/client-go/testing"
+	"k8s.io/utils/pointer"
 	eventingduckv1 "knative.dev/eventing/pkg/apis/duck/v1"
-	v1 "knative.dev/eventing/pkg/apis/duck/v1"
-	v1alpha1 "knative.dev/eventing/pkg/apis/duck/v1alpha1"
 	messagingv1 "knative.dev/eventing/pkg/apis/messaging/v1"
-	messagingv1beta1 "knative.dev/eventing/pkg/apis/messaging/v1beta1"
 	fakeeventingclient "knative.dev/eventing/pkg/client/injection/client/fake"
-	"knative.dev/eventing/pkg/client/injection/ducks/duck/v1alpha1/channelablecombined"
+	"knative.dev/eventing/pkg/client/injection/ducks/duck/v1/channelable"
 	channelreconciler "knative.dev/eventing/pkg/client/injection/reconciler/messaging/v1/channel"
 	"knative.dev/eventing/pkg/duck"
 	. "knative.dev/eventing/pkg/reconciler/testing/v1"
@@ -54,15 +52,15 @@ var (
 	testKey = fmt.Sprintf("%s/%s", testNS, channelName)
 
 	backingChannelHostname = network.GetServiceHostname("foo", "bar")
+
+	deliverySpec = &eventingduckv1.DeliverySpec{
+		Retry: pointer.Int32Ptr(10),
+	}
 )
 
 func init() {
 	// Add types to scheme
-	// TODO(nlopezgi): figure out what is the right list to have here.
-	_ = messagingv1beta1.AddToScheme(scheme.Scheme)
 	_ = messagingv1.AddToScheme(scheme.Scheme)
-	_ = v1.AddToScheme(scheme.Scheme)
-	_ = v1alpha1.AddToScheme(scheme.Scheme)
 	_ = eventingduckv1.AddToScheme(scheme.Scheme)
 }
 
@@ -88,6 +86,18 @@ func TestReconcile(t *testing.T) {
 				WithInitChannelConditions,
 				WithChannelDeleted),
 		},
+	}, {
+		Name: "Backing channel tracking error",
+		Key:  testKey,
+		Objects: []runtime.Object{
+			NewChannel(channelName, testNS,
+				WithChannelTemplate(channelCRDBadGvk()),
+				WithInitChannelConditions),
+		},
+		WantEvents: []string{
+			Eventf(corev1.EventTypeWarning, "InternalError", "unable to track changes to the backing Channel: %v:%v", "invalid Reference", "\nAPIVersion: name part must be non-empty\nAPIVersion: name part must consist of alphanumeric characters, '-', '_' or '.', and must start and end with an alphanumeric character (e.g. 'MyName',  or 'my.name',  or '123-abc', regex used for validation is '([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9]')\nKind: a valid C identifier must start with alphabetic character or '_', followed by a string of alphanumeric characters or '_' (e.g. 'my_name',  or 'MY_NAME',  or 'MyName', regex used for validation is '[A-Za-z_][A-Za-z0-9_]*')"),
+		},
+		WantErr: true,
 	}, {
 		Name: "Backing Channel.Create error",
 		Key:  testKey,
@@ -176,6 +186,77 @@ func TestReconcile(t *testing.T) {
 				WithChannelNoAddress(),
 				WithBackingChannelUnknown("BackingChannelNotConfigured", "BackingChannel has not yet been reconciled.")),
 		}},
+	}, {
+		Name: "Backing Kafka channel created",
+		Key:  testKey,
+		Objects: []runtime.Object{
+			NewChannel(channelName, testNS,
+				WithChannelTemplate(channelKafkaCRD()),
+				WithInitChannelConditions,
+				WithBackingChannelObjRef(backingKafkaChannelObjRef()),
+				WithBackingChannelReady,
+				WithChannelAddress(backingChannelHostname)),
+		},
+		WantCreates: []runtime.Object{
+			createKafkaChannel(testNS, channelName, false),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: NewChannel(channelName, testNS,
+				WithChannelTemplate(channelKafkaCRD()),
+				WithInitChannelConditions,
+				WithBackingChannelObjRef(backingKafkaChannelObjRef()),
+				WithChannelNoAddress(),
+				WithBackingChannelUnknown("BackingChannelNotConfigured", "BackingChannel has not yet been reconciled.")),
+		}},
+	}, {
+		Name: "Backing channel created with delivery",
+		Key:  testKey,
+		Objects: []runtime.Object{
+			NewChannel(channelName, testNS,
+				WithChannelTemplate(channelCRD()),
+				WithInitChannelConditions,
+				WithBackingChannelObjRef(backingChannelObjRef()),
+				WithBackingChannelReady,
+				WithChannelDelivery(deliverySpec),
+				WithChannelAddress(backingChannelHostname)),
+		},
+		WantCreates: []runtime.Object{
+			&unstructured.Unstructured{
+				Object: map[string]interface{}{
+					"apiVersion": "messaging.knative.dev/v1",
+					"kind":       "InMemoryChannel",
+					"metadata": map[string]interface{}{
+						"creationTimestamp": nil,
+						"namespace":         testNS,
+						"name":              channelName,
+						"ownerReferences": []interface{}{
+							map[string]interface{}{
+								"apiVersion":         "messaging.knative.dev/v1",
+								"blockOwnerDeletion": true,
+								"controller":         true,
+								"kind":               "Channel",
+								"name":               channelName,
+								"uid":                "",
+							},
+						},
+					},
+					"spec": map[string]interface{}{
+						"delivery": map[string]interface{}{
+							"retry": int64(10),
+						},
+					},
+				},
+			},
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: NewChannel(channelName, testNS,
+				WithChannelTemplate(channelCRD()),
+				WithInitChannelConditions,
+				WithBackingChannelObjRef(backingChannelObjRef()),
+				WithChannelNoAddress(),
+				WithChannelDelivery(deliverySpec),
+				WithBackingChannelUnknown("BackingChannelNotConfigured", "BackingChannel has not yet been reconciled.")),
+		}},
 	}, {
 		Name: "Generation Bump",
 		Key:  testKey,
@@ -241,11 +322,11 @@ func TestReconcile(t *testing.T) {
 
 	logger := logtesting.TestLogger(t)
 	table.Test(t, MakeFactory(func(ctx context.Context, listers *Listers, cmw configmap.Watcher) controller.Reconciler {
-		ctx = channelablecombined.WithDuck(ctx)
+		ctx = channelable.WithDuck(ctx)
 		r := &Reconciler{
 			dynamicClientSet:   fakedynamicclient.Get(ctx),
 			channelLister:      listers.GetMessagingChannelLister(),
-			channelableTracker: duck.NewListableTracker(ctx, channelablecombined.Get, func(types.NamespacedName) {}, 0),
+			channelableTracker: duck.NewListableTracker(ctx, channelable.Get, func(types.NamespacedName) {}, 0),
 		}
 		return channelreconciler.NewReconciler(ctx, logger,
 			fakeeventingclient.Get(ctx), listers.GetMessagingChannelLister(),
@@ -263,6 +344,20 @@ func channelCRD() metav1.TypeMeta {
 	}
 }
 
+func channelCRDBadGvk() metav1.TypeMeta {
+	return metav1.TypeMeta{
+		APIVersion: "",
+		Kind:       "",
+	}
+}
+
+func channelKafkaCRD() metav1.TypeMeta {
+	return metav1.TypeMeta{
+		APIVersion: "messaging.knative.dev/v1alpha1",
+		Kind:       "KafkaChannel",
+	}
+}
+
 func subscribers() []eventingduckv1.SubscriberSpec {
 	return []eventingduckv1.SubscriberSpec{{
 		UID:           "2f9b5e8e-deb6-11e8-9f32-f2801f1b9fd1",
@@ -331,6 +426,15 @@ func backingChannelObjRef() *duckv1.KReference {
 	}
 }
 
+func backingKafkaChannelObjRef() *duckv1.KReference {
+	return &duckv1.KReference{
+		APIVersion: "messaging.knative.dev/v1beta1",
+		Kind:       "KafkaChannel",
+		Namespace:  testNS,
+		Name:       channelName,
+	}
+}
+
 func createChannel(namespace, name string, ready bool) *unstructured.Unstructured {
 	var hostname string
 	var url string
@@ -386,3 +490,59 @@ func createChannel(namespace, name string, ready bool) *unstructured.Unstructure
 		},
 	}
 }
+
+func createKafkaChannel(namespace, name string, ready bool) *unstructured.Unstructured {
+	var hostname string
+	var url string
+	if ready {
+		return &unstructured.Unstructured{
+			Object: map[string]interface{}{
+				"apiVersion": "messaging.knative.dev/v1alpha1",
+				"kind":       "KafkaChannel",
+				"metadata": map[string]interface{}{
+					"creationTimestamp": nil,
+					"namespace":         namespace,
+					"name":              name,
+					"ownerReferences": []interface{}{
+						map[string]interface{}{
+							"apiVersion":         "messaging.knative.dev/v1",
+							"blockOwnerDeletion": true,
+							"controller":         true,
+							"kind":               "Channel",
+							"name":               name,
+							"uid":                "",
+						},
+					},
+				},
+				"status": map[string]interface{}{
+					"address": map[string]interface{}{
+						"hostname": hostname,
+						"url":      url,
+					},
+				},
+			},
+		}
+	}
+
+	return &unstructured.Unstructured{
+		Object: map[string]interface{}{
+			"apiVersion": "messaging.knative.dev/v1beta1",
+			"kind":       "KafkaChannel",
+			"metadata": map[string]interface{}{
+				"creationTimestamp": nil,
+				"namespace":         namespace,
+				"name":              name,
+				"ownerReferences": []interface{}{
+					map[string]interface{}{
+						"apiVersion":         "messaging.knative.dev/v1",
+						"blockOwnerDeletion": true,
+						"controller":         true,
+						"kind":               "Channel",
+						"name":               name,
+						"uid":                "",
+					},
+				},
+			},
+		},
+	}
+}
